package com.yaozhitech.spring5.config;

import java.time.Duration;

import org.springframework.beans.factory.ObjectProvider;
import org.springframework.cloud.circuitbreaker.resilience4j.ReactiveResilience4JCircuitBreakerFactory;
import org.springframework.cloud.circuitbreaker.resilience4j.Resilience4JConfigBuilder;
import org.springframework.cloud.client.circuitbreaker.Customizer;
import org.springframework.cloud.client.circuitbreaker.ReactiveCircuitBreakerFactory;
import org.springframework.cloud.gateway.filter.factory.SpringCloudCircuitBreakerFilterFactory;
import org.springframework.cloud.gateway.filter.factory.SpringCloudCircuitBreakerResilience4JFilterFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.reactive.DispatcherHandler;

import io.github.resilience4j.circuitbreaker.CircuitBreakerConfig;
import io.github.resilience4j.timelimiter.TimeLimiterConfig;

@Configuration
public class Resilience4JConfiguration {

	public Customizer<ReactiveResilience4JCircuitBreakerFactory> defaultCustomizer() {
	    return factory -> factory.configureDefault(id -> new Resilience4JConfigBuilder(id)
                .circuitBreakerConfig(CircuitBreakerConfig.custom().slowCallDurationThreshold(Duration.ofSeconds(4L)).build())
	            .timeLimiterConfig(TimeLimiterConfig.custom().timeoutDuration(Duration.ofSeconds(2)).build()).build());
	}

	public Customizer<ReactiveResilience4JCircuitBreakerFactory> slowCustomizer() {
	    return factory -> {
	        factory.configure(builder -> builder
	                .timeLimiterConfig(TimeLimiterConfig.custom().timeoutDuration(Duration.ofSeconds(4)).build())
	                .circuitBreakerConfig(CircuitBreakerConfig.ofDefaults()), "slow");
	    };
	}
	
	@Bean
    public SpringCloudCircuitBreakerFilterFactory resilience4JCircuitBreakerFactory(
            ReactiveCircuitBreakerFactory reactiveCircuitBreakerFactory,
            ObjectProvider<DispatcherHandler> dispatcherHandlers) {
		
		// 参考自 ReactiveResilience4JCustomizerConfiguration
		defaultCustomizer().customize((ReactiveResilience4JCircuitBreakerFactory) reactiveCircuitBreakerFactory);
		slowCustomizer().customize((ReactiveResilience4JCircuitBreakerFactory) reactiveCircuitBreakerFactory);
		
//		reactiveCircuitBreakerFactory.configureDefault(id -> new Resilience4JConfigBuilder((String) id)
//				.timeLimiterConfig(TimeLimiterConfig.custom().timeoutDuration(Duration.ofSeconds(12)).build()).build());
        return new SpringCloudCircuitBreakerResilience4JFilterFactory(reactiveCircuitBreakerFactory, 
        		dispatcherHandlers);
    }
}
